@@ -6,7 +6,7 @@ set.seed(290875)
 ### predict did not return factor levels for blackboost models
 dummy <- data.frame(y = gl(2, 100), x = runif(200))
 pr <- predict(blackboost(y ~ x, data = dummy, family = Binomial()),
-              newdata = dummy, type = "response")
+              newdata = dummy, type = "class")
 stopifnot(is.factor(pr) && all(levels(pr) %in% levels(dummy$y)))
 
 ### predict for g{al}mboost.matrix did not work
@@ -16,9 +16,11 @@ gb <- glmboost(x = X, y = dummy$y, family = Binomial(),
                control = ctrl)
 stopifnot(all.equal(predict(gb), predict(gb, newdata = X)))
 
+if (FALSE) {
 gb <- gamboost(x = X, y = dummy$y, family = Binomial(),
                control = ctrl)
 stopifnot(all.equal(predict(gb), predict(gb, newdata = X)))
+}
 
 ### blackboost _did_ touch the response, arg!
 
@@ -33,7 +35,7 @@ ctrl$risk <- "oobag"
 tmp <- blackboost(DEXfat ~ ., data = bodyfat, control = ctrl,
                  weights = bs[,3])
 
-stopifnot(identical(max(abs(tmp$risk[x] / sum(bs[,3] == 0)  - cv[3,])), 0))
+stopifnot(identical(max(abs(tmp$risk()[x] / sum(bs[,3] == 0)  - cv[3,])), 0))
 
 ### center = TRUE and cvrisk were broken; same issue with masking original data
 
@@ -42,7 +44,7 @@ cv1 <- cvrisk(gb, folds = bs)
 tmp <- glmboost(DEXfat ~ ., data = bodyfat,
                 control = boost_control(center = TRUE, risk = "oobag"),
                 weights = bs[,3])
-stopifnot(identical(max(tmp$risk[attr(cv1, "mstop")] / sum(bs[,3] == 0) - cv1[3,]), 0))
+stopifnot(identical(max(tmp$risk()[attr(cv1, "mstop")] / sum(bs[,3] == 0) - cv1[3,]), 0))
 
 ### same problem, just another check
 
@@ -68,6 +70,7 @@ gl <- glmboost(DEXfat ~ ., data = bodyfat, control = ctrl)
 stopifnot(max(abs(predict(ga) - predict(gl))) < 1e-8)
 AIC(gl)
 
+if (FALSE) {
 ### prediction with matrices was broken for gamboost,
 ### spotted by <Max.Kuhn@pfizer.com>
 x <- matrix(rnorm(1000), ncol = 10)
@@ -75,6 +78,7 @@ colnames(x) <- 1:ncol(x)
 y <- rnorm(100)
 fit <- gamboost(x = x, y = y, control = boost_control(mstop = 10))
 a <- predict(fit, newdata = x[1:10,])
+}
 
 ### AIC for centered covariates didn't work
 y <- gl(2, 10)
@@ -91,14 +95,14 @@ names(cg) <- NULL
 stopifnot(all.equal(cgc, cg))
 stopifnot(all.equal(mstop(AIC(gc, "classical")), mstop(AIC(g, "classical"))))
 
-gc <- gamboost(y ~ xn + xf, control = boost_control(center = TRUE),
+gc <- gamboost(y ~ xn + bols(xf), control = boost_control(center = TRUE),
                family = Binomial())
-g <- gamboost(y ~ xnm + xf, family = Binomial())
+g <- gamboost(y ~ xnm + bols(xf), family = Binomial())
 stopifnot(all.equal(mstop(AIC(gc, "classical")), mstop(AIC(g, "classical"))))
 
 y <- rnorm(20)
-gc <- gamboost(y ~ xn + xf, control = boost_control(center = TRUE))
-g <- gamboost(y ~ xnm + xf)
+gc <- gamboost(y ~ xn + bols(xf), control = boost_control(center = TRUE))
+g <- gamboost(y ~ xnm + bols(xf))
 stopifnot(all.equal(mstop(AIC(gc, "corrected")), mstop(AIC(g, "corrected"))))
 
 ### check gamboost with weights (use weighted some of residuals
@@ -134,7 +138,7 @@ stopifnot(all.equal(round(aic1, 1), round(aic2, 1)))
 mod1 <- blackboost(DEXfat ~ ., data = bodyfat, weights = w)
 mod2 <- blackboost(DEXfat ~ ., data = bodyfat[rep(1:n, w),])
 
-ratio <- mod1$risk / mod2$risk
+ratio <- mod1$risk() / mod2$risk()
 stopifnot(ratio[1] > 0.95 && ratio[2] < 1.05)
 
 ### df <= 2
@@ -142,13 +146,13 @@ ctrl$risk <- "oobag"
 mod1 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bss", dfbase = 2)
 mod2 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bbs", dfbase = 2)
 mod3 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bols")
-stopifnot(max(abs(predict(mod1) - predict(mod2))) < .Machine$double.eps)
-stopifnot(max(abs(predict(mod1) - predict(mod3))) < .Machine$double.eps)
+stopifnot(max(abs(predict(mod1) - predict(mod2))) < sqrt(.Machine$double.eps))
+stopifnot(max(abs(predict(mod1) - predict(mod3))) < sqrt(.Machine$double.eps))
 
 ### check predictions of zero-weight observations (via out-of-bag risk)
 mod1 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bss")
 mod2 <- gamboost(DEXfat ~ ., data = bodyfat, weights = w, con = ctrl, base = "bbs")
-stopifnot(abs(coef(lm(mod1$risk ~ mod2$risk - 1)) - 1) < 0.01)
+stopifnot(abs(coef(lm(mod1$risk() ~ mod2$risk() - 1)) - 1) < 0.01)
 
 ### not really a bug, a new feature; test fastp
 df <- data.frame(y = rnorm(100), x = runif(100), z = runif(100))
@@ -157,41 +161,15 @@ s <- seq(from = 1, to = 100, by = 3)
 
 x <- glmboost(y ~ ., data = df)
 for (i in s)
-    stopifnot(max(abs(predict(x[i]) - predict(x, all = TRUE)[,i])) < eps)
+    stopifnot(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)
 
 x <- gamboost(y ~ ., data = df)
 for (i in s)
-    stopifnot(max(abs(predict(x[i]) - predict(x, all = TRUE)[,i])) < eps)
+    stopifnot(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)
 
 x <- blackboost(y ~ ., data = df)
 for (i in s)
-    stopifnot(max(abs(predict(x[i]) - predict(x, all = TRUE)[,i])) < eps)
-
-### negative gradient of GaussClass was incorrectly specified
-### negative gradient of GaussClass was incorrectly specified
-data("BreastCancer", package = "mlbench")
-tmp <- BreastCancer[complete.cases(BreastCancer), -1]
-learn <- sample(1:nrow(tmp), ceiling(nrow(tmp) * 0.7))
-
-stump <- blackboost(Class ~ ., data = tmp[learn,],
-    tree_controls = ctree_control(teststat = "max",
-        testtype = "Teststatistic", mincriterion = 0, stump = TRUE),
-    control = boost_control(mstop = 176), family = GaussClass())
-mean(predict(stump, newdata = tmp[-learn,], type = "response") != tmp[-learn, "Class"])
-
-stump <- blackboost(Class ~ ., data = tmp[learn,],
-    tree_controls = ctree_control(teststat = "max",
-        testtype = "Teststatistic", mincriterion = 0, stump = TRUE),
-    control = boost_control(mstop = 275, constraint = TRUE), family = GaussClass())
-mean(predict(stump, newdata = tmp[-learn,], type = "response") != tmp[-learn, "Class"])
-
-cspline <- gamboost(Class ~ ., data = tmp[learn,],
-    control = boost_control(mstop = 126), family = GaussClass())
-mean(predict(cspline, newdata = tmp[-learn,], type = "response") != tmp[-learn, "Class"])
-
-cspline <- gamboost(Class ~ ., data = tmp[learn,],
-    control = boost_control(mstop = 73, constraint = FALSE), family = GaussClass())
-mean(predict(cspline, newdata = tmp[-learn,], type = "response") != tmp[-learn, "Class"])
+    stopifnot(max(abs(predict(x[i]) - predict(x[max(s)], agg = "cumsum")[,i])) < eps)
 
 ### make sure environment(formula) is used for evaluation
 data("cars")
@@ -199,10 +177,10 @@ ctl  <- boost_control(mstop = 100, trace = TRUE)
 tctl <- ctree_control(teststat = "max", testtype = "Teststat",
                       mincrit = 0, maxdepth = 5, savesplitstat = FALSE)
 myfun <- function(cars, xx, zz){
-  gamboost(dist ~ btree(speed, tree_controls = zz),
-           data = cars, control = xx)
+  mboost(dist ~ btree(speed, tree_controls = zz),
+         data = cars, control = xx)
 }
-mod <- myfun(cars, xx = ctl, zz = tctl)
+try(mod <- myfun(cars, xx = ctl, zz = tctl))
 
 ### bbs with weights and expanded data
 x <- runif(100)
@@ -211,35 +189,104 @@ knots <- seq(from = 0.1, to = 0.9, by = 0.1)
 w <- rmultinom(1, length(x), rep(1, length(x)) / length(x))[,1]
 iw <- rep(1:length(x), w)
 
-m1 <- attr(bbs(x, knots = knots), "dpp")(w)$fit(y)$model
-m2 <- attr(bbs(x[iw], knots = knots), "dpp")(rep(1, length(x)))$fit(y[iw])$model
+m1 <- bbs(x, knots = knots)$dpp(w)$fit(y)$model
+m2 <- bbs(x[iw], knots = knots)$dpp(rep(1, length(iw)))$fit(y[iw])$model
 
 stopifnot(max(abs(m1 - m2)) < sqrt(.Machine$double.eps))
 
+### base learner handling
+stopifnot(max(abs(fitted(gamboost(DEXfat ~ age, data = bodyfat)) -
+                  fitted(gamboost(DEXfat ~ bbs(age), data = bodyfat)))) <
+          sqrt(.Machine$double.eps))
+
+### predict for matrix interface to glmboost
+x <- matrix(runif(1000), ncol = 10)
+y <- rowMeans(x) + rnorm(nrow(x))
+mod <- glmboost(x = x, y = y)
+stopifnot(length(predict(mod, newdata = x[1:2,])) == 2)
+try(predict(mod, newdata = as.data.frame(x[1:2,])))
+
+
 ### predict for varying coefficient models with categorical z
-## (coercion of factor to design matrix did not work)
 set.seed(1907)
 x <- rnorm(100)
 z <- gl(2,50)
 y <- rnorm(100, sd = 0.1)
 data <- data.frame(y=y, x=x, z=z)
 
-model <- gamboost(y ~ bols(x, z=z), data = data)
+model <- gamboost(y ~ bols(x, by=z), data = data)
 stopifnot(!is.na(predict(model,data[1,-1])))
 
-model <- gamboost(y ~ bbs(x, z=z), data = data)
-stopifnot(!is.na(predict(model,data[1,-1])))
-
-model <- gamboost(y ~ bns(x, z=z), data = data)
+model <- gamboost(y ~ bbs(x, by=z), data = data)
 stopifnot(!is.na(predict(model,data[1,-1])))
 
 x <- as.factor(sample(1:10, 100, replace=TRUE))
 data <- data.frame(y=y, x=x, z=z)
-model <- gamboost(y ~ brandom(x, z=z), data = data)
+model <- gamboost(y ~ brandom(x, by=z), data = data)
 stopifnot(!is.na(predict(model,data[1,-1])))
 
 x1 <- rnorm(100)
 x2 <- rnorm(100)
 data <- data.frame(y=y, x1=x1, x2=x2, z=z)
-model <- gamboost(y ~ bspatial(x1,x2, z=z), data = data)
+model <- gamboost(y ~ bspatial(x1,x2, by=z), data = data)
 stopifnot(!is.na(predict(model,data[1,-1])))
+
+### bols with intercept = FALSE for categorical covariates was broken
+x <- gl(2, 50)
+y <- c(rnorm(50, mean = -1), rnorm(50, mean = 1))
+stopifnot(length(coef(gamboost(y ~ bols(x, intercept=FALSE)))) == 1)
+
+# check also for conntinuous x
+x <- rnorm(100)
+y <- c(rnorm(100, mean = 1 * x))
+stopifnot(length(coef(gamboost(y ~ bols(x, intercept=FALSE)))) == 1)
+
+### check interface of coef
+set.seed(1907)
+x1 <- rnorm(100)
+int <- rep(1, 100)
+y <- 3 * x1 + rnorm(100, sd=0.1)
+dummy <- data.frame(y = y, int = int, x1 = x1)
+
+gbm <- gamboost(y ~ bols(int, intercept=FALSE) +  bols(x1, intercept=FALSE) + bbs(x1, center=TRUE, df=1), data = dummy)
+
+### check prediction if intercept=FALSE
+gbm <- gamboost(y ~ bols(x1, intercept=FALSE), data = dummy)
+stopifnot(!is.na(predict(gbm)) & max(abs(predict(gbm) - fitted(gbm))) < sqrt(.Machine$double.eps))
+
+### check coef.glmboost
+set.seed(1907)
+x1 <- rnorm(100)
+x2 <- rnorm(100)
+y <- rnorm(100, mean= 3 * x1,sd=0.01)
+linMod <- glmboost(y ~ x1 + x2)
+stopifnot(length(coef(linMod)) == 2)
+
+### automated offset computation in Family
+x <- rnorm(100)
+y <- rnorm(100)
+w <- drop(rmultinom(1, 100, rep(1 / 100, 100)))
+
+G <- Gaussian()
+fm <- Family(ngradient = G@ngradient, risk = G@risk)
+
+m1 <- glmboost(y ~ x, family = G)
+m2 <- glmboost(y ~ x, family = fm)
+stopifnot(all.equal(coef(m1), coef(m2)))
+
+### formula evaluation
+f <- function() {
+
+    x <- rnorm(100)
+    y <- rnorm(100)
+    list(mboost(y ~ bbs(x)),
+         gamboost(y ~ x),
+         glmboost(y ~ x),
+         blackboost(y ~ x))
+}
+tmp <- f()
+
+### both loss and risk given, spotted by
+### Fabian Scheipl <fabian.scheipl@stat.uni-muenchen.de>
+stopifnot(extends(class(Family(ngradient = function(y, f) y, loss = function(y, w) sum(y), 
+       risk = function(y, f, w) sum(y))), "boost_family")
